const fs = require("fs");
const path = require("path");

const { to10DigitTimestamp } = require("./timestamp.js");

const LAST_FETCH_FILENAME = "last-fetch.txt";
const TOKEN_FILENAME = "token.json";
const ACTIVITIES_DIR = "activities";
const stravaPath = path.join(process.cwd(), ".strava");
const tokenPath = path.join(stravaPath, TOKEN_FILENAME);

const isTokenValid = token => {
  if (token && token.client_id && token.client_secret && token.access_token && token.refresh_token && token.expires_at) {
    return true;
  }

  return false;
};

class Cache {
  constructor() {
    if (!fs.existsSync(stravaPath)) {
      fs.mkdirSync(stravaPath);
    }

    this.lastFetchPath = path.join(stravaPath, LAST_FETCH_FILENAME);
    this.activitiesPath = path.join(stravaPath, ACTIVITIES_DIR);

    if (!fs.existsSync(this.activitiesPath)) {
      fs.mkdirSync(this.activitiesPath);
    }

    this.token = null;
  }

  getActivities() {
    const activities = [];

    const fileNames = fs.readdirSync(this.activitiesPath);

    fileNames.forEach(fileName => {
      const activity = JSON.parse(fs.readFileSync(path.join(this.activitiesPath, fileName), "utf-8"));

      activities.push(activity);
    });

    return activities;
  }

  setActivity(activity) {
    fs.writeFileSync(path.join(this.activitiesPath, `${activity.id}.json`), JSON.stringify(activity));
  }

  getLastFetch() {
    if (!fs.existsSync(this.lastFetchPath)) {
      return null;
    }

    const lastFetched = fs.readFileSync(this.lastFetchPath, "utf-8");
    return to10DigitTimestamp(lastFetched);
  }

  setLastFetch(newLastFetch) {
    const oldLastFetched = this.getLastFetch();

    if (newLastFetch > oldLastFetched) {
      fs.writeFileSync(this.lastFetchPath, to10DigitTimestamp(newLastFetch));
    }
  }

  getToken() {
    if (this.token) {
      return this.token;
    }

    let token;

    if (process.env.GATSBY_SOURCE_STRAVA_TOKEN) {
      token = JSON.parse(process.env.GATSBY_SOURCE_STRAVA_TOKEN);
    }

    if (!token) {
      throw new Error("No token. Please generate one using `gatsby-source-strava-token` command");
    }

    if (!isTokenValid(token)) {
      throw new Error("Invalid token. Please regenerate one using `gatsby-source-strava-token` command");
    }

    this.token = token;

    return this.token;
  }

  setToken(token) {
    if (isTokenValid(token)) {
      this.token = token;
    }

    // fs.writeFileSync(
    //   tokenPath,
    //   JSON.stringify({
    //     client_id,
    //     client_secret,
    //     access_token,
    //     refresh_token,
    //     expires_at,
    //     expires_in,
    //   })
    // )
  }
}

let cache = new Cache();

module.exports = {
  cache
};